#
# Unit Tests for Elements of a Model
#
# TestSimpleVar                Class for testing single variables
# TestArrayVar                Class for testing array of variables
# TestSimpleObj                Class for testing single objective
# TestArrayObj                Class for testing array of objective
# TestSimpleCon                Class for testing single constraint
# TestArrayCon                Class for testing array of constraint
# TestModel             Class to test the Model class
#

import unittest
import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+"..")
currdir = dirname(abspath(__file__))+os.sep

from coopr.pyomo.base import IntegerSet
from coopr.pyomo import *
from coopr.opt import *
from coopr.pyomo.base.var import _VarElement
import pyutilib.th
import pyutilib.services

class PyomoModel(pyutilib.th.TestCase):

    def setUp(self):
        self.model = Model()

    def construct(self,filename):
        self.instance = self.model.create(filename)


class TestSimpleVar(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        pass

    def test_fixed_attr(self):
        """Test fixed attribute"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.instance.x.fixed = True
        self.failUnlessEqual(self.instance.x.fixed, True)

    def test_value_attr(self):
        """Test value attribute"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.instance.x.value = 3.5
        self.failUnlessEqual(self.instance.x.value, 3.5)

    def test_initial_attr(self):
        """Test initial attribute"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.instance.x.initial = 3.5
        self.failUnlessEqual(self.instance.x.initial, 3.5)

    def test_domain_attr(self):
        """Test domain attribute"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.instance.x.domain = Integers
        self.failUnlessEqual(type(self.instance.x.domain), IntegerSet)

    def test_name_attr(self):
        """Test name attribute"""
        #
        # A user would never need to do this, but this 
        # attribute is needed within Pyomo
        #
        self.model.x = Var()
        self.model.x.name = "foo"
        self.failUnlessEqual(self.model.x.name, "foo")

    def test_lb_attr1(self):
        """Test lb attribute"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.instance.x.lb = -1.0
        self.failUnlessEqual(self.instance.x.lb, -1.0)

    def test_lb_attr2(self):
        """Test lb attribute"""
        self.model.x = Var(within=NonNegativeReals, bounds=(-1,2))
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.lb, 0.0)
        self.failUnlessEqual(self.instance.x.ub, 2.0)

    def test_ub_attr1(self):
        """Test ub attribute"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.instance.x.ub = 1.0
        self.failUnlessEqual(self.instance.x.ub, 1.0)

    def test_ub_attr2(self):
        """Test ub attribute"""
        self.model.x = Var(within=NonPositiveReals, bounds=(-2,1))
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.lb, -2.0)
        self.failUnlessEqual(self.instance.x.ub, 0.0)

    def test_within_option(self):
        """Test within option"""
        self.model.x = Var(within=Integers)
        self.failUnlessEqual(type(self.model.x.domain), IntegerSet)

    def test_initialize_option(self):
        """Test initialize option"""
        self.model.x = Var(initialize=1.3)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.initial, 1.3)

    def test_bounds_option1(self):
        """Test bounds option"""
        def x_bounds(model):
          return (-1.0,1.0)
        self.model.x = Var(bounds=x_bounds)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.lb, -1.0)
        self.failUnlessEqual(self.instance.x.ub, 1.0)

    def test_bounds_option2(self):
        """Test bounds option"""
        self.model.x = Var(bounds=(-1.0,1.0))
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.lb, -1.0)
        self.failUnlessEqual(self.instance.x.ub, 1.0)

    def test_rule_option(self):
        """Test rule option"""
        def x_init(model):
          return 1.3
        self.model.x = Var(initialize=x_init)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.value, None)
        self.failUnlessEqual(self.instance.x.initial, 1.3)

    def test_reset(self):
        """Test reset method"""
        self.model.x = Var(initialize=3)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.initial,3)
        self.failUnlessEqual(self.instance.x.value,None)
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.x.initial,3)
        self.failUnlessEqual(self.instance.x.initial,3)

    def test_dim(self):
        """Test dim method"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.dim(),0)

    def test_keys(self):
        """Test keys method"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.keys(),[None])

    def test_len(self):
        """Test len method"""
        self.model.x = Var()
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.x),1)

    def test_simplify(self):
        """Test simplify method"""
        self.model.x = Var()
        self.failUnlessEqual(type(self.model.x.simplify(self.model)),_VarElement)

    def test_value(self):
        """Check the value of the variable"""
        self.model.x = Var(initialize=3.3)
        self.instance = self.model.create()
        tmp = value(self.instance.x.initial)
        self.failUnlessEqual( type(tmp), float)
        self.failUnlessEqual( tmp, 3.3 )
        tmp = float(self.instance.x.initial)
        self.failUnlessEqual( type(tmp), float)
        self.failUnlessEqual( tmp, 3.3 )
        tmp = int(self.instance.x.initial)
        self.failUnlessEqual( type(tmp), int)
        self.failUnlessEqual( tmp, 3 )


class TestArrayVar(TestSimpleVar):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2])

    def test_fixed_attr(self):
        """Test fixed attribute"""
        self.model.x = Var(self.model.A)
        self.model.y = Var(self.model.A)
        self.instance = self.model.create()
        self.instance.x.fixed = True
        #try:
          #self.instance.x.fixed
        #except AttributeError:
          #pass
        #else:
          #self.fail("test_fixed_attr")
        self.failUnlessEqual(self.instance.x[1].fixed, False)
        self.instance.y[1].fixed=True
        self.failUnlessEqual(self.instance.y[1].fixed, True)

    def test_value_attr(self):
        """Test value attribute"""
        self.model.x = Var(self.model.A)
        self.model.y = Var(self.model.A)
        self.instance = self.model.create()
        try:
            self.instance.x = 3.5
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.y[1] = 3.5
        self.failUnlessEqual(self.instance.y[1], 3.5)

    #def test_initial_attr(self):
        #"""Test initial attribute"""
        #self.model.x = Var(self.model.A)
        #self.instance = self.model.create()
        #self.instance.x.initial = 3.5
        #self.failUnlessEqual(self.instance.x[1].initial, 3.5)

    #def test_lb_attr(self):
        #"""Test lb attribute"""
        #self.model.x = Var(self.model.A)
        #self.instance = self.model.create()
        #self.instance.x.lb = -1.0
        #self.failUnlessEqual(self.instance.x[1].lb, -1.0)

    #def test_ub_attr(self):
        #"""Test ub attribute"""
        #self.model.x = Var(self.model.A)
        #self.instance = self.model.create()
        #self.instance.x.ub = 1.0
        #self.failUnlessEqual(self.instance.x[1].ub, 1.0)

    def test_initialize_option(self):
        """Test initialize option"""
        self.model.x = Var(self.model.A,initialize={1:1.3,2:2.3})
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1], 1.3)
        self.failUnlessEqual(self.instance.x[2], 2.3)

    def test_bounds_option1(self):
        """Test bounds option"""
        def x_bounds(i,model):
          return (-1.0,1.0)
        self.model.x = Var(self.model.A, bounds=x_bounds)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1].lb, -1.0)
        self.failUnlessEqual(self.instance.x[1].ub, 1.0)

    def test_bounds_option2(self):
        """Test bounds option"""
        self.model.x = Var(self.model.A, bounds=(-1.0,1.0))
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1].lb, -1.0)
        self.failUnlessEqual(self.instance.x[1].ub, 1.0)

    def test_rule_option(self):
        """Test rule option"""
        def x_init(i,model):
          return 1.3
        self.model.x = Var(self.model.A, initialize=x_init)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1].initial, 1.3)

    def test_reset(self):
        """Test reset method"""
        self.model.x = Var(self.model.A,initialize=3)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1].initial,3)
        self.failUnlessEqual(self.instance.x[1].value,None)
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.x[1].initial,3)
        self.failUnlessEqual(self.instance.x[1].initial,3)

    def test_dim(self):
        """Test dim method"""
        self.model.x = Var(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.dim(),1)

    def test_keys(self):
        """Test keys method"""
        self.model.x = Var(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.keys(),[1,2])

    def test_len(self):
        """Test len method"""
        self.model.x = Var(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.x),2)

    def test_value(self):
        """Check the value of the variable"""
        self.model.x = Var(self.model.A,initialize=3.3)
        self.instance = self.model.create()
        tmp = value(self.instance.x[1].initial)
        self.failUnlessEqual( type(tmp), float)
        self.failUnlessEqual( tmp, 3.3 )
        tmp = float(self.instance.x[1].initial)
        self.failUnlessEqual( type(tmp), float)
        self.failUnlessEqual( tmp, 3.3 )
        tmp = int(self.instance.x[1].initial)
        self.failUnlessEqual( type(tmp), int)
        self.failUnlessEqual( tmp, 3 )


class Test2DArrayVar(TestSimpleVar):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2])

    def test_fixed_attr(self):
        """Test fixed attribute"""
        self.model.x = Var(self.model.A,self.model.A)
        self.model.y = Var(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.instance.x.fixed = True
        #try:
          #self.instance.x.fixed
        #except AttributeError:
          #pass
        #else:
          #self.fail("test_fixed_attr")
        self.failUnlessEqual(self.instance.x[1,2].fixed, False)
        self.instance.y[1,2].fixed=True
        self.failUnlessEqual(self.instance.y[1,2].fixed, True)

    def test_value_attr(self):
        """Test value attribute"""
        self.model.x = Var(self.model.A,self.model.A)
        self.model.y = Var(self.model.A,self.model.A)
        self.instance = self.model.create()
        try:
            self.instance.x = 3.5
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.y[1,2] = 3.5
        self.failUnlessEqual(self.instance.y[1,2], 3.5)

    #def test_initial_attr(self):
        #"""Test initial attribute"""
        #self.model.x = Var(self.model.A,self.model.A)
        #self.instance = self.model.create()
        #self.instance.x.initial = 3.5
        #self.failUnlessEqual(self.instance.x[1,1].initial, 3.5)

    #def test_lb_attr(self):
        #"""Test lb attribute"""
        #self.model.x = Var(self.model.A,self.model.A)
        #self.instance = self.model.create()
        #self.instance.x.lb = -1.0
        #self.failUnlessEqual(self.instance.x[2,1].lb, -1.0)

    #def test_ub_attr(self):
        #"""Test ub attribute"""
        #self.model.x = Var(self.model.A,self.model.A)
        #self.instance = self.model.create()
        #self.instance.x.ub = 1.0
        #self.failUnlessEqual(self.instance.x[2,1].ub, 1.0)

    def test_initialize_option(self):
        """Test initialize option"""
        self.model.x = Var(self.model.A,self.model.A,initialize={(1,1):1.3,(2,2):2.3})
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1,1], 1.3)
        self.failUnlessEqual(self.instance.x[2,2], 2.3)
        try:
            value(self.instance.x[1,2])
            self.fail("Expected KeyError")
        except KeyError:
            pass

    def test_bounds_option1(self):
        """Test bounds option"""
        def x_bounds(i,j,model):
          return (-1.0*(i+j),1.0*(i+j))
        self.model.x = Var(self.model.A, self.model.A, bounds=x_bounds)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1,1].lb, -2.0)
        self.failUnlessEqual(self.instance.x[1,2].ub, 3.0)

    def test_bounds_option2(self):
        """Test bounds option"""
        self.model.x = Var(self.model.A, self.model.A, bounds=(-1.0,1.0))
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1,1].lb, -1.0)
        self.failUnlessEqual(self.instance.x[1,1].ub, 1.0)

    def test_rule_option(self):
        """Test rule option"""
        def x_init(i,j,model):
          return 1.3
        self.model.x = Var(self.model.A, self.model.A, initialize=x_init)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1,2].initial, 1.3)

    def test_reset(self):
        """Test reset method"""
        self.model.x = Var(self.model.A,self.model.A, initialize=3)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x[1,1].initial,3)
        self.failUnlessEqual(self.instance.x[1,1].value,None)
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.x[1,1].initial,3)
        self.failUnlessEqual(self.instance.x[1,1].initial,3)

    def test_dim(self):
        """Test dim method"""
        self.model.x = Var(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.x.dim(),2)

    def test_keys(self):
        """Test keys method"""
        self.model.x = Var(self.model.A,self.model.A)
        self.instance = self.model.create()
        ans = [(1,1),(1,2),(2,1),(2,2)]
        self.failUnlessEqual(self.instance.x.keys().sort(),ans.sort())

    def test_len(self):
        """Test len method"""
        self.model.x = Var(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.x),4)

    def test_value(self):
        """Check the value of the variable"""
        self.model.x = Var(self.model.A,self.model.A,initialize=3.3)
        self.instance = self.model.create()
        tmp = value(self.instance.x[1,1].initial)
        self.failUnlessEqual( type(tmp), float)
        self.failUnlessEqual( tmp, 3.3 )
        tmp = float(self.instance.x[1,1].initial)
        self.failUnlessEqual( type(tmp), float)
        self.failUnlessEqual( tmp, 3.3 )
        tmp = int(self.instance.x[1,1].initial)
        self.failUnlessEqual( type(tmp), int)
        self.failUnlessEqual( tmp, 3 )


class TestSimpleObj(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        pass

    def test_rule_option(self):
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(rule=f)
        self.instance = self.model.create()
        try:
          self.failUnlessEqual(self.instance.obj(), 8)
        except ValueError:
          pass
        else:
          self.fail("test_rule_option")
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.obj(), 8)
        self.failUnlessEqual(value(self.instance.obj), 8)

    def test_sense_option(self):
        """Test sense option"""
        def rule(model):
            return 1.0
        self.model.obj = Objective(sense=maximize, rule=rule)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.obj.sense, maximize)

    def test_dim(self):
        """Test dim method"""
        def rule(model):
            return 1
        self.model.obj = Objective(rule=rule)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.obj.dim(),0)

    def test_keys(self):
        """Test keys method"""
        def rule(model):
            return 1
        self.model.obj = Objective(rule=rule)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.obj.keys(),[None])

    def test_len(self):
        """Test len method"""
        def rule(model):
            return 1.0
        self.model.obj = Objective(rule=rule)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.obj),1)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(rule=f)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.obj),1)


class TestArrayObj(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2])

    def tearDown(self):
        pass

    def test_rule_option(self):
        """Test rule option"""
        def f(i,model):
          ans=0
          for j in model.x.keys():
            ans = ans + model.x[j]
          ans *= i
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(self.model.A,rule=f)
        self.instance = self.model.create()
        try:
            self.instance.obj[1]()
            self.fail("Excepted ValueError due to uninitialized variables")
        except ValueError:
            pass
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.obj[1](), 8)
        self.failUnlessEqual(self.instance.obj[2](), 16)
        self.failUnlessEqual(value(self.instance.obj[1]), 8)
        self.failUnlessEqual(value(self.instance.obj[2]), 16)

    def test_sense_option(self):
        """Test sense option"""
        self.model.obj = Objective(self.model.A,sense=maximize)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.obj.sense, maximize)

    def test_dim(self):
        """Test dim method"""
        self.model.obj = Objective(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.obj.dim(),1)

    def test_keys(self):
        """Test keys method"""
        self.model.obj = Objective(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.obj.keys()),2)

    def test_len(self):
        """Test len method"""
        self.model.obj = Objective(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.obj),0)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(rule=f)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.obj),1)


class Test2DArrayObj(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2])

    def tearDown(self):
        pass

    def test_rule_option(self):
        """Test rule option"""
        def f(i,k,model):
          ans=0
          for j in model.x.keys():
            ans = ans + model.x[j]
          ans *= i
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(self.model.A,self.model.A, rule=f)
        self.instance = self.model.create()
        try:
            self.failUnlessEqual(self.instance.obj(),None)
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.obj[1,1](), 8)
        self.failUnlessEqual(self.instance.obj[2,1](), 16)
        self.failUnlessEqual(value(self.instance.obj[1,1]), 8)
        self.failUnlessEqual(value(self.instance.obj[2,1]), 16)

    def test_sense_option(self):
        """Test sense option"""
        self.model.obj = Objective(self.model.A,self.model.A,sense=maximize)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.obj.sense, maximize)

    def test_dim(self):
        """Test dim method"""
        self.model.obj = Objective(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.obj.dim(),2)

    def test_keys(self):
        """Test keys method"""
        self.model.obj = Objective(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.obj.keys()),4)

    def test_len(self):
        """Test len method"""
        self.model.obj = Objective(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.obj),0)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.obj = Objective(rule=f)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.obj),1)


class TestSimpleCon(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        pass

    def test_rule1(self):
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          ans = ans >= 0
          ans = ans <= 1
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        try:
          self.failUnlessEqual(self.instance.c(), 8)
        except ValueError:
          pass
        else:
          self.fail("test_rule_option")
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c(), 8)
        self.failUnlessEqual(value(self.instance.c), 8)

    def test_rule2(self):
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return (0,ans,1)
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        try:
          self.failUnlessEqual(self.instance.c(), 8)
        except ValueError:
          pass
        else:
          self.fail("test_rule_option")
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c(), 8)
        self.failUnlessEqual(value(self.instance.c), 8)

    def test_rule3(self):
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return (0,ans,None)
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        try:
          self.failUnlessEqual(self.instance.c(), 8)
        except ValueError:
          pass
        else:
          self.fail("test_rule_option")
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c(), 8)
        self.failUnlessEqual(value(self.instance.c), 8)

    def test_rule4(self):
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return (None,ans,1)
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        try:
          self.failUnlessEqual(self.instance.c(), 8)
        except ValueError:
          pass
        else:
          self.fail("test_rule_option")
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c(), 8)
        self.failUnlessEqual(value(self.instance.c), 8)

    def test_rule5(self):
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          return (ans,1)
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        try:
          self.failUnlessEqual(self.instance.c(), 8)
        except ValueError:
          pass
        else:
          self.fail("test_rule_option")
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c(), 8)
        self.failUnlessEqual(value(self.instance.c), 8)

    def test_dim(self):
        """Test dim method"""
        self.model.c = Constraint()
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.c.dim(),0)

    def test_keys(self):
        """Test keys method"""
        self.model.c = Constraint()
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.c.keys(),[])

    def test_len(self):
        """Test len method"""
        self.model.c = Constraint()
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.c),0)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          ans = ans == 0
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.c),1)


class TestArrayCon(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2,3,4])

    def tearDown(self):
        pass

    def test_rule_option1(self):
        """Test rule option"""
        def f(i,model):
          ans=0
          for j in model.x.keys():
            ans = ans + model.x[j]
          ans *= i
          ans = ans < 0
          ans = ans > 0
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        try:
            self.instance.c[1]()
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c[1](), 8)
        self.failUnlessEqual(self.instance.c[2](), 16)
        self.failUnlessEqual(value(self.instance.c[1]), 8)
        self.failUnlessEqual(value(self.instance.c[2]), 16)
        self.failUnlessEqual(len(self.instance.c), 4)

    def test_rule_option2(self):
        """Test rule option"""
        def f(i,model):
          if i%2 == 0:
                return 0
          ans=0
          for j in model.x.keys():
            ans = ans + model.x[j]
          ans *= i
          ans = ans < 0
          ans = ans > 0
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        try:
            self.instance.c[1]()
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c[1](), 8)
        self.failUnlessEqual(value(self.instance.c[1]), 8)
        self.failUnlessEqual(len(self.instance.c), 2)

    def test_rule_option3(self):
        """Test rule option"""
        def f(i,model):
          if i%2 == 0:
                return None
          ans=0
          for j in model.x.keys():
            ans = ans + model.x[j]
          ans *= i
          ans = ans < 0
          ans = ans > 0
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        try:
            self.instance.c[1]()
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c[1](), 8)
        self.failUnlessEqual(value(self.instance.c[1]), 8)
        self.failUnlessEqual(len(self.instance.c), 2)

    def test_rule_option4(self):
        """Test rule option"""
        def f(model):
          res={}
          for i in [0,1,2]:
                ans=0
                for j in model.x.keys():
                    ans = ans + model.x[j]
                ans *= i
                ans = ans < 0
                ans = ans > 0
                res[i]=ans
          return res
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        try:
            self.instance = self.model.create()
            self.fail("Expected IndexError")
        except IndexError:
            pass

    def test_dim(self):
        """Test dim method"""
        self.model.c = Constraint(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.c.dim(),1)

    def test_keys(self):
        """Test keys method"""
        self.model.c = Constraint(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.c.keys()),0)

    def test_len(self):
        """Test len method"""
        self.model.c = Constraint(self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.c),0)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          ans = ans==2
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.c),1)

    def test_rule_error1(self):
        """Verify that errors in the rule dictionary are identified"""
        def f(model):
          res={}
          for i in model.A:
            if i%2 != 0:
                ans=0
                for j in model.x.keys():
                    ans = ans + model.x[j]
                ans *= i
                ans = ans < 0
                ans = ans > 0
                res[i]=ans
          return res
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        try:
            self.instance.c[1]()
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c[1](), 8)
        self.failUnlessEqual(value(self.instance.c[1]), 8)
        self.failUnlessEqual(len(self.instance.c), 2)



class Test2DArrayCon(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2])

    def tearDown(self):
        pass

    def test_rule_option(self):
        """Test rule option"""
        def f(i,j,model):
          ans=0
          for j in model.x.keys():
            ans = ans + model.x[j]
          ans *= i
          ans = ans < 0
          ans = ans > 0
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,self.model.A,rule=f)
        self.instance = self.model.create()
        try:
            self.instance.c[1,1]()
            self.fail("Expected ValueError")
        except ValueError:
            pass
        self.instance.x.reset()
        self.failUnlessEqual(self.instance.c[1,1](), 8)
        self.failUnlessEqual(self.instance.c[2,1](), 16)
        self.failUnlessEqual(value(self.instance.c[1,1]), 8)
        self.failUnlessEqual(value(self.instance.c[2,1]), 16)

    def test_dim(self):
        """Test dim method"""
        self.model.c = Constraint(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(self.instance.c.dim(),2)

    def test_keys(self):
        """Test keys method"""
        self.model.c = Constraint(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.c.keys()),0)

    def test_len(self):
        """Test len method"""
        self.model.c = Constraint(self.model.A,self.model.A)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.c),0)
        """Test rule option"""
        def f(model):
          ans=0
          for i in model.x.keys():
            ans = ans + model.x[i]
          ans = ans==2
          return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.failUnlessEqual(len(self.instance.c),1)


class TestModel(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        #pass
        if os.path.exists("unknown.lp"):
           os.unlink("unknown.lp")

    def test_clear_attribute(self):
        """ Coverage of the _clear_attribute method """
        self.model.A = Set()
        self.failUnlessEqual(self.model.A.name,"A")
        self.model.A = Var()
        self.failUnlessEqual(self.model.A.name,"A")
        self.model.A = Param()
        self.failUnlessEqual(self.model.A.name,"A")
        self.model.A = Objective()
        self.failUnlessEqual(self.model.A.name,"A")
        self.model.A = Constraint()
        self.failUnlessEqual(self.model.A.name,"A")
        self.model.A = Set()
        self.failUnlessEqual(self.model.A.name,"A")

    def test_set_attr(self):
        self.model.x = Param()
        self.model.x = None

    def test_write(self):
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
          expr = 0
          for i in model.A:
            expr += model.x[i]
          return expr
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        self.instance.write()

    def test_write2(self):
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
          expr = 0
          for i in model.A:
            expr += model.x[i]
          return expr
        self.model.obj = Objective(rule=obj_rule)
        def c_rule(model):
          return (1, model.x[1]+model.x[2], 2)
        self.model.c = Constraint(rule=c_rule)
        self.instance = self.model.create()
        self.instance.write()

    def test_write_error(self):
        self.instance = self.model.create()
        try:
            self.instance.write(format=ProblemFormat.ospl)
            self.fail("Expected error because ospl writer is not available")
        except ValueError:
            pass

    def test_solve1(self):
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
          expr = 0
          for i in model.A:
            expr += model.x[i]
          return expr
        self.model.obj = Objective(rule=obj_rule)
        def c_rule(model):
          expr = 0
          for i in model.A:
            expr += i*model.x[i]
          return (expr,0)
        self.model.c = Constraint(rule=c_rule)
        self.instance = self.model.create()
        #self.instance.pprint()
        if not pyutilib.services.registered_executable("glpsol"):
            return
        else:   
           opt = SolverFactory('glpk')
        solutions = opt.solve(self.instance, keepFiles=True)
        self.instance.load(solutions)
        self.instance.display(currdir+"solve1.out")
        self.failUnlessFileEqualsBaseline(currdir+"solve1.out",currdir+"solve1.txt")

    def Xtest_solve2(self):
        """
        WEH - this is disabled because glpk appears to work fine
        on this example.  I'm not quite sure what has changed that has
        impacted this test...
        """
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
          expr = 0
          for i in model.A:
            expr += model.x[i]
          return expr
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        #self.instance.pprint()
        if not pyutilib.services.registered_executable("glpsol"):
            return
        else:   
           opt = solvers.GLPK(keepFiles=True)
        solutions = opt.solve(self.instance)
        solutions.write()
        sys.exit(1)
        try:
            self.instance.load(solutions)
            self.fail("Cannot load a solution with a bad solver status")
        except ValueError:
            pass

    def test_solve3(self):
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
          expr = 0
          for i in model.A:
            expr += model.x[i]
          return expr
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        self.instance.display(currdir+"solve3.out")
        self.failUnlessFileEqualsBaseline(currdir+"solve3.out",currdir+"solve3.txt")

    def test_solve4(self):
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
          expr = 0
          for i in model.A:
            expr += model.x[i]
          return expr
        self.model.obj = Objective(rule=obj_rule)
        def c_rule(model):
          expr = 0
          for i in model.A:
            expr += i*model.x[i]
          return (expr,0)
        self.model.c = Constraint(rule=c_rule)
        self.instance = self.model.create()
        #self.instance.pprint()
        if not pyutilib.services.registered_executable("glpsol"):
            return
        else:   
           opt = SolverFactory('glpk', keepFiles=True)
        solutions = opt.solve(self.instance)
        self.instance.load(solutions.solution(0))
        self.instance.display(currdir+"solve1.out")
        self.failUnlessFileEqualsBaseline(currdir+"solve1.out",currdir+"solve1.txt")

if __name__ == "__main__":
   unittest.main()

